@@ -7,12 +7,12 @@ heroku.models
 This module contains the models that comprise the Heroku API.
 """
 
-import json
-from urllib import quote
-
-import requests
 from .helpers import to_python
 from .structures import *
+from urllib import quote
+import json
+import requests
+
 
 
 class BaseResource(object):
@@ -120,15 +120,16 @@ class Addon(AvailableAddon):
         )
         return r.ok
 
-    def new(self, name):
+    def new(self, name, params=None):
         r = self._h._http_resource(
             method='POST',
-            resource=('apps', self.app.name, 'addons', name)
+            resource=('apps', self.app.name, 'addons', name),
+            params=params
         )
         r.raise_for_status()
         return self.app.addons[name]
 
-    def upgrade(self, name):
+    def upgrade(self, name, params=None):
         """Upgrades an addon to the given tier."""
         # Allow non-namespaced upgrades. (e.g. advanced vs logging:advanced)
         if ':' not in name:
@@ -137,6 +138,7 @@ class Addon(AvailableAddon):
         r = self._h._http_resource(
             method='PUT',
             resource=('apps', self.app.name, 'addons', quote(name)),
+            params=params,
             data=' '   # Server weirdness.
         )
         r.raise_for_status()
@@ -233,6 +235,13 @@ class App(BaseResource):
             resource=('apps', self.name),
             obj=App,
         )
+        
+    @property
+    def labs(self):
+        return self._h._get_resources(
+            resource=('features'),
+            obj=Feature, params={'app': self.name}, app=self, map=filtered_key_list_resource_factory(lambda item: item.kind == 'app')
+        )
 
     def rollback(self, release):
         """Rolls back the release to the given version."""
@@ -306,7 +315,7 @@ class App(BaseResource):
         )
 
         # Grab the actual logs.
-        r = requests.get(r.content, verify=False)
+        r = requests.get(r.content, verify=False, prefetch=False)
 
         if not tail:
             return r.content
@@ -451,8 +460,6 @@ class Key(BaseResource):
             data=key
         )
 
-        print key.split()[-1]
-
         return self._h.keys.get(key.split()[-1])
 
     def delete(self):
@@ -480,7 +487,7 @@ class Process(BaseResource):
 
     _ints = ['elapsed']
     _bools = ['attached']
-    _dates = ['transitioned_at']
+    _dates = []
     _pks = ['process', 'upid']
 
 
@@ -491,6 +498,17 @@ class Process(BaseResource):
     def __repr__(self):
         return "<process '{0}'>".format(self.process)
 
+    def new(self, type, quantity):
+        r = self._h._http_resource(
+            method='POST',
+            resource=('apps', self.app.name, 'ps', 'scale'),
+            data={'type': type, 'qty': quantity}
+        )
+
+        r.raise_for_status()
+
+        return self.app.processes[type]
+
     @property
     def type(self):
         return self.process.split('.')[0]
@@ -569,3 +587,30 @@ class Stack(BaseResource):
         super(Stack, self).__init__()
 
 
+class Feature(BaseResource):
+    _strs = ['name', 'kind', 'summary', 'docs',]
+    _bools = ['enabled']
+    _pks = ['name']
+    
+    def __init__(self):
+        self.app = None
+        super(Feature, self).__init__()
+
+    def __repr__(self):
+        return "<feature '{0}'>".format(self.name)
+    
+    def enable(self):
+        r = self._h._http_resource(
+            method='POST',
+            resource=('features', self.name),
+            params={'app': self.app.name if self.app else ''}
+        )
+        return r.ok
+    
+    def disable(self):
+        r = self._h._http_resource(
+            method='DELETE',
+            resource=('features', self.name),
+            params={'app': self.app.name if self.app else ''}
+        )
+        return r.ok
